--
-- Mundunugu Caddy
--

local mundunugu      	= CreateFrame("Frame","MundunuguFrame",UIParent)
local mundunugu_options	= CreateFrame("Frame","MundunuguOptions")

-- accounting variables

local ver="3.1"

-- frame buttons
local fire_b	={}
local earth_b	={}
local water_b	={}
local air_b		={}

-- visible spell frames

local fire_f
local earth_f
local water_f
local air_f

-- saved variables
Mundunugu_vars =	{	fx,fy,fp,
						ex,ey,ep,
						wx,wy,wp,
						ax,ay,ap,
						
						scale,
						frl,erl,wrl,arl
					}
					
Mundunugu_opts =	{	scale,
						frl,erl,wrl,arl
					}
--


function mundunugu:new()
-- this is only for shaman
-- if you aren't a shaman, just don't do anything

	if not (UnitClass("player")=="Shaman") then
		return
	end

-- mundunuguframe is the main frame for receiving messages
-- it is not to be visible
-- the individual spell frames will be moveable and visible

	self:SetToplevel(false)
	self:SetWidth(10)
	self:SetHeight(10)

	self:RegisterEvent("VARIABLES_LOADED")
	self:RegisterEvent("PLAYER_LOGIN")
	self:RegisterEvent("SPELLS_CHANGED")
	self:SetScript("OnEvent",self.processevent)
	self:SetPoint("TOPRIGHT",UIParent)
	DEFAULT_CHAT_FRAME:AddMessage("Mundunugu Caddy "..ver.." initialized.")
end

function mundunugu_mousedown(self,button)
	if button=="RightButton" then
		self:StartMoving()
	end
end

function mundunugu_mouseup(self,button)
	if button=="RightButton" then
		local p,rt,rp,x,y=self:GetPoint()
		self:StopMovingOrSizing()
		Mundunugu_vars.fp,_,_,Mundunugu_vars.fx,Mundunugu_vars.fy=fire_f:GetPoint()
		Mundunugu_vars.ep,_,_,Mundunugu_vars.ex,Mundunugu_vars.ey=earth_f:GetPoint()
		Mundunugu_vars.wp,_,_,Mundunugu_vars.wx,Mundunugu_vars.wy=water_f:GetPoint()
		Mundunugu_vars.ap,_,_,Mundunugu_vars.ax,Mundunugu_vars.ay=air_f:GetPoint()
	end
end

function mundunugu:processevent(event,...)
	if event=="PLAYER_LOGIN" then
		self:buildframes()

		-- intiate the options panel
		mundunugu_init_optionspanel()
	end

-- if the player is in combat, we'll get bizarre behavior (or even errors) so lets not try
-- anything while in combat.  This might happen because SPELLS_CHANGED happens when the
-- player opens the spell book as well as when new spells come in and even when the player
-- changes weapons.  There is no way to know which of those events triggered the event.

-- the only time we really care about changing the buttons is when the player learns a new spell.
-- this is never going to happen while in combat, so we can just use UnitAffectingCombat() to
-- keep bizarre hide/show behavior from happening by doing nothing.

-- Why bother with this?  Because this will let the addon change buttons if you learn a new spell.

	if event=="SPELLS_CHANGED" then
		if not UnitAffectingCombat("player") then
			self:buildframes()
		end
	end
	
	if event=="VARIABLES_LOADED" then
		if Mundunugu_vars.fx==nil then Mundunugu_vars.fx=0 end
		if Mundunugu_vars.fy==nil then Mundunugu_vars.fy=0 end
		if Mundunugu_vars.fp==nil then Mundunugu_vars.fp="CENTER" end
		if Mundunugu_vars.ex==nil then Mundunugu_vars.ex=0 end
		if Mundunugu_vars.ey==nil then Mundunugu_vars.ey=0 end
		if Mundunugu_vars.ep==nil then Mundunugu_vars.ep="CENTER" end
		if Mundunugu_vars.wx==nil then Mundunugu_vars.wx=0 end
		if Mundunugu_vars.wy==nil then Mundunugu_vars.wy=0 end
		if Mundunugu_vars.wp==nil then Mundunugu_vars.wp="CENTER" end
		if Mundunugu_vars.ax==nil then Mundunugu_vars.ax=0 end
		if Mundunugu_vars.ay==nil then Mundunugu_vars.ay=0 end
		if Mundunugu_vars.ap==nil then Mundunugu_vars.ap="CENTER" end
		if Mundunugu_vars.scale==nil or Mundunugu_vars.scale==0 then Mundunugu_vars.scale=1.0 end
		if Mundunugu_vars.frl==nil then Mundunugu_vars.frl=1 end
		if Mundunugu_vars.erl==nil then Mundunugu_vars.erl=1 end
		if Mundunugu_vars.wrl==nil then Mundunugu_vars.wrl=1 end
		if Mundunugu_vars.arl==nil then Mundunugu_vars.arl=1 end
	end
end

--
-- Mundunugu Options Stuff
--

--
-- Called when the scale slider is moved
--

function mundunugu_scale()
	local s=mundunugu_options.scaleslide:GetValue()
	
	Mundunugu_vars.scale=s	
	fire_f:SetScale(s)
	earth_f:SetScale(s)
	water_f:SetScale(s)
	air_f:SetScale(s)
end

--
-- create the customization gadgets in the options panel
--

function mundunugu_init_optionspanel()
	local scaleslide
	
	mundunugu_options.name="Mundunugu Caddy"
	mundunugu_options.okay=mundunugu_options_ok
	mundunugu_options.cancel=mundunugu_options_cancel
	mundunugu_options:SetScript("OnShow",mundunugu_options_show)

-- Create a scale slider

	mundunugu_options.scaleslide=CreateFrame("Slider","mundunugu_scaleslide",mundunugu_options,"OptionsSliderTemplate")
	scaleslide=mundunugu_options.scaleslide
	scaleslide:SetWidth(100)
	scaleslide:SetHeight(16)
	scaleslide:SetPoint("TOPLEFT",mundunugu_options,20,-50)
	getglobal(scaleslide:GetName().."Text"):SetText("Elemental Frame Scale")
	getglobal(scaleslide:GetName().."High"):SetText("1.0")
	getglobal(scaleslide:GetName().."Low"):SetText("0.4")
	scaleslide:SetScript("OnValueChanged",mundunugu_scale)
	
-- create checkbox for horde/ally totem title images
	
--	mundunugu_options.hora=CreateFrame("checkbutton","mundunugu_hora",mundunugu_options,"OptionsCheckButtonTemplate")
	
--	local hora=mundunugu_options.hora
--	hora:SetPoint("TOPLEFT",mundunugu_options,20,-70)
--	hora:SetWidth(30)
--	hora:SetHeight(30)
--	hora:SetScript("OnClick",mundunugu_horaclick)
	
-- create checkboxes for img orientation

	mundunugu_options.f_orient=CreateFrame("CheckButton","mundunugu_frl",mundunugu_options,"OptionsCheckButtonTemplate")
	mundunugu_options.e_orient=CreateFrame("CheckButton","mundunugu_erl",mundunugu_options,"OptionsCheckButtonTemplate")
	mundunugu_options.w_orient=CreateFrame("CheckButton","mundunugu_wrl",mundunugu_options,"OptionsCheckButtonTemplate")
	mundunugu_options.a_orient=CreateFrame("CheckButton","mundunugu_arl",mundunugu_options,"OptionsCheckButtonTemplate")
	
	local f_orient=mundunugu_options.f_orient
	local e_orient=mundunugu_options.e_orient
	local w_orient=mundunugu_options.w_orient
	local a_orient=mundunugu_options.a_orient
	
	f_orient:SetWidth(30)
	f_orient:SetHeight(30)
	e_orient:SetWidth(30)
	e_orient:SetHeight(30)
	w_orient:SetWidth(30)
	w_orient:SetHeight(30)
	a_orient:SetWidth(30)
	a_orient:SetHeight(30)
	
	f_orient:SetPoint("TOPLEFT",mundunugu_options,20,-90)
	e_orient:SetPoint("TOPLEFT",mundunugu_options,20,-110)
	w_orient:SetPoint("TOPLEFT",mundunugu_options,20,-130)
	a_orient:SetPoint("TOPLEFT",mundunugu_options,20,-150)
	
	f_orient:SetScript("OnClick",mundunugu_orientclicked)
	e_orient:SetScript("OnClick",mundunugu_orientclicked)
	w_orient:SetScript("OnClick",mundunugu_orientclicked)
	a_orient:SetScript("OnClick",mundunugu_orientclicked)

-- make a copy of the settings so we can be properly handled in the options frame
	Mundunugu_opts.scale=Mundunugu_vars.scale
	Mundunugu_opts.frl=Mundunugu_vars.frl
	Mundunugu_opts.erl=Mundunugu_vars.erl
	Mundunugu_opts.wrl=Mundunugu_vars.wrl
	Mundunugu_opts.arl=Mundunugu_vars.arl

	mundunugu_options:SetScript("OnEvent",mundunugu_options_processevent)
	InterfaceOptions_AddCategory(mundunugu_options)
end

--
-- adjust the options buttons accordingly if we enter/leave combat while its open
--

function mundunugu_options_processevent(self,event,...)
	if event=="PLAYER_REGEN_DISABLED" then
		mundunugu_options:UnregisterEvent("PLAYER_REGEN_DISABLED")
		mundunugu_options:RegisterEvent("PLAYER_REGEN_ENABLED")
		mundunugu_options_disableall()
	end
	
	if event=="PLAYER_REGEN_ENABLED" then
		mundunugu_options:UnregisterEvent("PLAYER_REGEN_ENABLED")
		mundunugu_options:RegisterEvent("PLAYER_REGEN_DISABLED")
		mundunugu_options_enableall()
	end
end

--
-- one of the right/left title image orientation buttons were clicked
--

function mundunugu_orientclicked(self)
	local orienttext = {"Right","Left"}
	local fname = {"fire","earth","water","air"}
	local crl=self:GetChecked()
	local i
	
	if crl then
		i=2
	else
		i=1
	end
	
	getglobal(self:GetName().."Text"):SetText(orienttext[i].." align "..fname[i].." frame image")
	mundunugu_softupdate()
end

--
-- this is called to reflect changes made by the options window.
-- we don't want to permanently change anything but all of our
-- draw functions use the global stored variables
-- in order to not have to make a copy of these functions
-- we just just make a local copy, change the global settings, call
-- the draw function then put the saved copy back into the global settings
--

function mundunugu_softupdate()
	local saverl={1,1,1,1}
	
	if mundunugu_options.f_orient:GetChecked() then
		Mundunugu_vars.frl=2
	else
		Mundunugu_vars.frl=1
	end
	if mundunugu_options.e_orient:GetChecked() then
		Mundunugu_vars.erl=2
	else
		Mundunugu_vars.erl=1
	end
	if mundunugu_options.w_orient:GetChecked() then
		Mundunugu_vars.wrl=2
	else
		Mundunugu_vars.wrl=1
	end
	if mundunugu_options.a_orient:GetChecked() then
		Mundunugu_vars.arl=2
	else
		Mundunugu_vars.arl=1
	end
	
	mundunugu:buildframes()
end

--
-- called if we cancel the changes in the options interface
--

function mundunugu_options_cancel()
	local s=Mundunugu_opts.scale
	
	Mundunugu_vars.scale=s
	Mundunugu_vars.frl=Mundunugu_opts.frl
	Mundunugu_vars.erl=Mundunugu_opts.erl
	Mundunugu_vars.wrl=Mundunugu_opts.wrl
	Mundunugu_vars.arl=Mundunugu_opts.arl

	mundunugu:buildframes()

	fire_f:SetScale(s)
	earth_f:SetScale(s)
	water_f:SetScale(s)
	air_f:SetScale(s)

	mundunugu_options:UnregisterAllEvents()
end

--
-- called if we press OK in the options interface
--

function mundunugu_options_ok()
	-- make it so #1
	Mundunugu_opts.scale=Mundundugu_vars.scale
	
	Mundunugu_opts.frl=Mundunugu_vars.frl
	Mundunugu_opts.erl=Mundunugu_vars.erl
	Mundunugu_opts.wrl=Mundunugu_vars.wrl
	Mundunugu_opts.arl=Mundunugu_vars.arl

	mundunugu_options:UnregisterAllEvents()
end

--
-- this is called whenever the options window appears
-- we want the controls to reflect current settings
--

function mundunugu_options_show()
	local orienttext = {"Right","Left"}
	local imgtext = {"Horde","Alliance"}

	if UnitAffectingCombat("player") then
		mundunugu_options_disableall()
		mundunugu_options:RegisterEvent("PLAYER_REGEN_ENABLED")
	else
		mundunugu_options_enableall()
		mundunugu_options:RegisterEvent("PLAYER_REGEN_DISABLED")
	end
	
	-- set the values of the options buttons/sliders
	mundunugu_options.scaleslide:SetValue(Mundunugu_vars.scale)

	mundunugu_options.f_orient:SetChecked((Mundunugu_vars.frl==2))
	mundunugu_options.e_orient:SetChecked((Mundunugu_vars.erl==2))
	mundunugu_options.w_orient:SetChecked((Mundunugu_vars.wrl==2))
	mundunugu_options.a_orient:SetChecked((Mundunugu_vars.arl==2))

	getglobal(mundunugu_options.f_orient:GetName().."Text"):SetText(orienttext[Mundunugu_vars.frl].." align fire frame image")
	getglobal(mundunugu_options.e_orient:GetName().."Text"):SetText(orienttext[Mundunugu_vars.erl].." align earth frame image")
	getglobal(mundunugu_options.w_orient:GetName().."Text"):SetText(orienttext[Mundunugu_vars.wrl].." align water frame image")
	getglobal(mundunugu_options.a_orient:GetName().."Text"):SetText(orienttext[Mundunugu_vars.arl].." align air frame image")
	
end

--
-- call this to disable the options settings
-- used when we are in combat and the options window is open
--

function mundunugu_options_disableall()
-- effectively disable the slider
	local scaleslide=mundunugu_options.scaleslide
	local s=scaleslide:GetValue()
	
	scaleslide:SetMinMaxValues(s,s)
	scaleslide:SetValueStep(0)

-- gray out the buttons

	mundunugu_options.f_orient:Disable()
	mundunugu_options.e_orient:Disable()
	mundunugu_options.w_orient:Disable()
	mundunugu_options.a_orient:Disable()
end

--
-- call this to restore the functions of the options window
--

function mundunugu_options_enableall()
	local scaleslide=mundunugu_options.scaleslide

	scaleslide:SetMinMaxValues(0.4,1.0)
	scaleslide:SetValueStep(0.05)

	mundunugu_options.f_orient:Enable()
	mundunugu_options.e_orient:Enable()
	mundunugu_options.w_orient:Enable()
	mundunugu_options.a_orient:Enable()
end

--
-----------------------------------
--end options stuff
-----------------------------------
--

--
-- addon functions
--

function mundunugu:makeframe(framename)
	local frame
	
	frame=CreateFrame("Frame",framename,UIParent)
	
	frame:SetToplevel(true)
	frame:EnableMouse(true)
	frame:SetMovable(true)
	frame:SetWidth(185)
	frame:SetHeight(66)

	frame:SetBackdrop({
				bgFile="Interface\\Tooltips\\UI-Tooltip-Background",
				edgeFile="Interface\\Tooltips\\UI-Tooltip-Border",
				tile=1, tileSize=10, edgeSize=10,
				insets={left=3, right=3, top=3, bottom=3}
				})
	frame:SetBackdropColor(0,0,0,.50)
	frame:SetBackdropBorderColor(0,0,0,0)
	frame:SetScript("OnMouseDown",mundunugu_mousedown)
	frame:SetScript("OnMouseUp",mundunugu_mouseup)
	frame:SetScale(Mundunugu_vars.scale)
	frame:Show()
	return frame
end

--
-- applytitleimage()
-- frame is a valid frame pointer
-- icon is a path to the image to put in the title area
-- framename is a name to insert into frame name
-- w is the width of the image
-- h is the height of the image
-- xoff is the x offset of the image
-- yoff is the y offset of the image
-- creates and shows a title icon for the given frame
--

function mundunugu:applytitleimage(frame,icon,framename,w,h,xoff,yoff)
-- create a button, we aren't really going to make it do anything
	if frame.titleimg==nil then
		frame.titleimg=CreateFrame("Button","mundunugu"..framename.."title",frame)
		frame.titleimg:SetWidth(w)
		frame.titleimg:SetHeight(h)
	end
	frame.titleimg:SetNormalTexture(icon)
	frame.titleimg:GetNormalTexture():SetTexCoord(0,1,0,0.515625)
	frame.titleimg:ClearAllPoints()
	frame.titleimg:SetPoint("TOPLEFT",frame,xoff,yoff)
	frame.titleimg:SetScript("OnMouseDown",mundunugu_titlebutton_mousedown)
	frame.titleimg:SetScript("OnMouseUp",mundunugu_titlebutton_mouseup)
	frame.titleimg:Show()
end

--
-- Call these functions if the title "button" is dragged so that the entire
-- frame is dragged.
-- We could copy mundunugu_mousedown/up but if we ever changed those functions
-- We would have to remember to also change these.
-- In this way, we only have to change one function and let these call the generic functions.
--

function mundunugu_titlebutton_mousedown(self,button)
	if button=="RightButton" then
		mundunugu_mousedown(self:GetParent(),button)
	end
end

function mundunugu_titlebutton_mouseup(self,button)
	if button=="RightButton" then
		mundunugu_mouseup(self:GetParent(),button)
	end
end

function mundunugu:buildframes()

-- global spellIDs for the fire spells
					
	local fire_spellIDs	={	25489,25457,25557,25533,25552,
							25547,25560,2894}

	local fires			=8

	local foffx			={	{32,62,92,122,152,
							 32,62,       152},
							{2,32,62,92,122,
							 2,32,      122}
						}
					
	local foffy			={	-3,-3,-3,-3,-3,
							-33,-33,    -33}

-- global spellIDs for the earth spells

	local earth_spellIDs	={	10399,25454,25509,8143,25525,
								25528,2484,32594,2062}

	local earths			=9
	
	local eoffx				={	{32,62,92,122,152,
								 32,62,   122,152},
								{2,32,62,92,122,
								 2,32,   92,122}
							}
					
	local eoffy				={	-3,-3,-3,-3,-3,
								-33,-33,  -33,-33}

-- global spellIDs for the water spells

	local water_spellIDs	={	25500,25464,25567,25570,8170,
								16190,25563,33736}
	local waters			=8

	local woffx				={	{32,62,92,122,152,
								 32,62,92},
								{2,32,62,92,122,
								 2,32,62}
							}
					
	local woffy				={	-3,-3,-3,-3,-3,
								-33,-33,-33}

-- global spellIDs for the air spells

	local air_spellIDs	={		25505,57994,8177,8512,6495,
								25574,3738}
	local airs			=7

	local aoffx			={	{32,62,92,122,152,
							 62,122},
							{2,32,62,92,122,
							 32,92}
						}
					

	local aoffy			={	-3,-3,-3,-3,-3,
							-33,-33}

	local imgoff			={ 3,152 }
	
-- create the fire frame
	if fire_f == nil then
		fire_f=self:makeframe("fireframe")
		fire_f:ClearAllPoints()
		fire_f:SetPoint(Mundunugu_vars.fp,UIParent,Mundunugu_vars.fx,Mundunugu_vars.fy)
	end
	self:applytitleimage(fire_f,"Interface\\Addons\\Mundunugu\\Icons\\fire.tga",
					"fire",29,60,imgoff[Mundunugu_vars.frl],-3)
	self:framebuttons(fire_f,fire_b,fire_spellIDs,fires,foffx[Mundunugu_vars.frl],foffy,"fire")
	
-- create the earth frame
	if earth_f == nil then
		earth_f=self:makeframe("earthframe")
		earth_f:ClearAllPoints()
		earth_f:SetPoint(Mundunugu_vars.ep,UIParent,Mundunugu_vars.ex,Mundunugu_vars.ey)
	end
	self:applytitleimage(earth_f,"Interface\\Addons\\Mundunugu\\Icons\\earth.tga",
					"earth",29,60,imgoff[Mundunugu_vars.erl],0)
	self:framebuttons(earth_f,earth_b,earth_spellIDs,earths,eoffx[Mundunugu_vars.erl],eoffy,"earth")

-- create the water frame
	if water_f == nil then
		water_f=self:makeframe("waterframe")
		water_f:ClearAllPoints()
		water_f:SetPoint(Mundunugu_vars.wp,UIParent,Mundunugu_vars.wx,Mundunugu_vars.wy)
	end
	self:applytitleimage(water_f,"Interface\\Addons\\Mundunugu\\Icons\\water.tga",
					"water",29,60,imgoff[Mundunugu_vars.wrl],0)
	self:framebuttons(water_f,water_b,water_spellIDs,waters,woffx[Mundunugu_vars.wrl],woffy,"water")

-- create the air frame
	if air_f == nil then
		air_f=self:makeframe("airframe")
		air_f:ClearAllPoints()
		air_f:SetPoint(Mundunugu_vars.ap,UIParent,Mundunugu_vars.ax,Mundunugu_vars.ay)
	end

	self:applytitleimage(air_f,"Interface\\Addons\\Mundunugu\\Icons\\air.tga",
					"air",29,60,imgoff[Mundunugu_vars.arl],0)
	self:framebuttons(air_f,air_b,air_spellIDs,airs,aoffx[Mundunugu_vars.arl],aoffy,"air")

-- add in a check for improved wind fury totem (Wow 3.0 release)

	local wind=getglobal("mundunuguairframebutton3")
	
	if wind then
		local name,rank,icon,bookID=self:GetSpellInBook(29193) -- 29193 is improved wind fury totem highest rank

		wind:SetNormalTexture(icon)
		wind:SetPushedTexture(icon)

		wind:GetNormalTexture():SetWidth(wind:GetWidth())
		wind:GetNormalTexture():SetHeight(wind:GetHeight())

		wind:GetPushedTexture():SetWidth(W)
		wind:GetPushedTexture():SetHeight(H)
		wind.tipID=bookID
	end
end

function mundunugu_buttonevent(thisone,event,arg1,arg2,...)
	local t
	local start,dur,enable
	local name
	local cooldown

-- I have no idea why this works
-- check that the event in question is SPELL_UPDATE_COOLDOWN
	if event=="SPELL_UPDATE_COOLDOWN" then

-- some code I found in SpellBookFrame.lua  no clue why I can only make cooldownframe_settimer
-- work only this way
		name=thisone:GetName()
		cooldown=getglobal(name.."Cooldown")

-- all examples I saw only use "thisone" as the button/frame to be working with apparently that
-- is wrong as the only way I can get it to work was to get the global name of name.."Cooldown"

		start,dur,enable=GetSpellCooldown(thisone:GetAttribute("spell"))
		CooldownFrame_SetTimer(cooldown,start,dur,enable)
	end
end

function mundunugu_tooltipon(self)
-- there is only one tooltip and its name is GameTooltip
	
	GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )

-- set the spell name so we get the ingame spell info associated with this button
-- this is why we grabbed the spellbook id in the enumeration process and made
-- sure we had the right rank
-- I created a new member of the button table just for this purpose (tipID)

	GameTooltip:SetSpell(self.tipID,BOOKTYPE_SPELL)
	GameTooltip:Show()
end

function mundunugu_tooltipoff(self)
	GameTooltip:Hide()
end

--
-- returns the name, rank, icon and spellbook ID of the spell with global id of 'spellID'
-- all of these are nil if the spell doesn't exist in the current player's spellbook
--

function mundunugu:GetSpellInBook(spellID)
	local gname
	local name
	local icon
	local rname,rrank,rbid,ricon

	rname=nil
	rrank=nil
	rbid=nil
	ricon=nil
	
	if spellID==nil then
		DEFAULT_CHAT_FRAME:AddMessage("Mundunugu: Attempting to index a nil spellID.")
		return nil,nil,nil,nil
	end

	gname=GetSpellInfo(spellID)
	if gname == nil then
		DEFAULT_CHAT_FRAME:AddMessage("Mundunugu: "..spellID.." not found.")
		return nil,nil,nil,nil
	end
	name,rank,icon=GetSpellInfo(gname)

	if name then
		rname=name
		ricon=icon
		rrank=rank
		rbid=mundunugu_GetSpellbookID(name,rank)
	end
	return rname,rrank,ricon,rbid
end

--
-- framebuttons()
-- input:  frame is a pointer to a valid frame
--         button is an array of button pointers
--         spellID is an array of spellIDs to enumerate
--         n is the number of elements in the arrays passed to this function
--         offx is an array of x offsets from topleft of the parent frame
--         offy is an array of y offsets from topleft of the parent frame
--         framename is a name for the frame that will be inserted into the global name for the
--             newly created buttons
--         ttID is an array of tooltip spellbook IDs for use with the GameTooltip function
--
-- output: nothing
--
-- this function will create and attach an array of buttons to a given frame.  the buttons are created
-- only if the spells in the spellID array are in the player's spellbook. button[] and ttID[] are
-- filled with appropriate values for use elsewhere
--

function mundunugu:framebuttons(frame,button,spellID,n,offx,offy,framename,ttID)
	local t

	for t = 1,n do
		local name,rank,icon,bookID
		
		name,rank,icon,bookID=self:GetSpellInBook(spellID[t])
		if name then
			if button[t] == nil then
				button[t] = CreateFrame("Button","mundunugu"..framename.."button"..t,frame,"ActionButtonTemplate,SecureActionButtonTemplate")

		-- Gather its width and height because the textures are larger than the button as given
		-- this way we can set those sizes right and make all the other buttons match the size	

				W = button[t]:GetWidth()*.8
				H = button[t]:GetHeight()*.8
			else
				W = button[t]:GetWidth()
				H = button[t]:GetHeight()
			end

		-- Set type and spell for each state
			button[t]:SetAttribute("type","spell")
			button[t]:SetAttribute("spell",name)

		-- Set the image for the button
			button[t]:SetNormalTexture(icon)
			button[t]:SetPushedTexture(icon)

			button[t]:SetWidth(W)
			button[t]:SetHeight(H)
		
			button[t]:GetNormalTexture():SetWidth(W)
			button[t]:GetNormalTexture():SetHeight(H)

			button[t]:GetPushedTexture():SetWidth(W)
			button[t]:GetPushedTexture():SetHeight(H)
	
			button[t]:SetPoint("TOPLEFT",frame,offx[t],offy[t])
			button[t]:RegisterEvent("SPELL_UPDATE_COOLDOWN")
			button[t]:SetScript("OnEvent",mundunugu_buttonevent)
			button[t]:SetScript("OnEnter",mundunugu_tooltipon)
			button[t]:SetScript("OnLeave",mundunugu_tooltipoff)

			button[t].tipID=bookID
			
			button[t]:Show()
		end
	end
end

--
-- GetSpellbookID()
-- input:  name is a valid spell name, rank is a valid spell rank
-- output: the player's spellbook ID where the spell 'name' of rank 'rank' is located at
-- 

function mundunugu_GetSpellbookID(name,rank)
	local i
	local id=nil
	
	i=1
	while true do
		local n,r=GetSpellName(i,BOOKTYPE_SPELL)
		if (n==name) and (r==rank) then
			id=i
			break
		end
	i = i + 1
	end
	return id
end

mundunugu:new()
